package org.fax4j.spi.windows;

import org.fax4j.FaxJob;
import org.fax4j.FaxJobStatus;
import org.fax4j.common.FaxActionType;
import org.fax4j.spi.FaxClientSpi;
import org.fax4j.spi.process.ProcessOutputHandler;
import org.fax4j.spi.windows.WindowsProcessFaxClientSpi.Fax4jExeConstants;
import org.fax4j.util.ProcessExecutorHelper.ProcessOutput;

/**
 * This class implements the windows process output handler which is
 * used to update the fax job based on the process output information.
 * 
 * @author 	Sagie Gur-Ari
 * @version 1.0
 * @since	0.41.5
 */
public class WindowsProcessOutputHandler implements ProcessOutputHandler
{
	/**
	 * This is the class constructor.
	 */
	public WindowsProcessOutputHandler()
	{
		super();
	}
	
	/**
	 * This function returns the relevant part from the process output.
	 * 
	 * @param 	processOutput
	 * 			The process output
	 * @param 	prefix
	 * 			The prefix to look for in the output
	 * @return	The relevant output part
	 */
	protected String getOutputPart(ProcessOutput processOutput,String prefix)
	{
		//get output
		String output=processOutput.getOutputText();

		if(output!=null)
		{
			//set flag
			boolean validOutput=false;

			//output is fax job ID
			int index=output.indexOf(prefix);
			if(index!=-1)
			{
				//get index
				index=index+prefix.length();
	
				if(output.length()>index)
				{
					//get output
					output=output.substring(index);
					
					output=output.trim();
					index=output.indexOf("\n");
					if(index!=-1)
					{
						output=output.substring(0,index);
						output=output.trim();
					}

					if(output.length()>0)
					{
						//set flag
						validOutput=true;
					}
				}
			}
			
			if(!validOutput)
			{
				output=null;
			}
		}
		
		return output;
	}

	/**
	 * Updates the fax job based on the data from the process output.
	 *  
	 * @param	faxClientSpi
	 * 			The fax client SPI
	 * @param 	faxJob
	 * 			The fax job object
	 * @param 	processOutput
	 * 			The process output
	 * @param	faxActionType
	 * 			The fax action type
	 */
	public void updateFaxJob(FaxClientSpi faxClientSpi,FaxJob faxJob,ProcessOutput processOutput,FaxActionType faxActionType)
	{
		//get output
		String output=this.getOutputPart(processOutput,Fax4jExeConstants.FAX_JOB_ID_OUTPUT_PREFIX);

		if(output!=null)
		{
			//validate fax job ID
			WindowsFaxClientSpiHelper.validateFaxJobID(output);
			
			//set fax job ID
			faxJob.setID(output);
		}
	}
	
	/**
	 * This function extracts the fax job status from the process output.
	 * 
	 * @param	faxClientSpi
	 * 			The fax client SPI
	 * @param	processOutput
	 * 			The process output
	 * @return	The fax job status
	 */
	public FaxJobStatus getFaxJobStatus(FaxClientSpi faxClientSpi,ProcessOutput processOutput)
	{
		//get output
		String output=this.getOutputPart(processOutput,Fax4jExeConstants.FAX_JOB_STATUS_OUTPUT_PREFIX);

		FaxJobStatus faxJobStatus=null;
		if(output!=null)
		{
			//get fax job status
			faxJobStatus=WindowsFaxClientSpiHelper.getFaxJobStatusFromWindowsFaxJobStatusString(output);
		}

		return faxJobStatus;
	}
}